@@ -7,6 +7,8 @@
 // TODO:
 #define _(x) (x)
 
+extern RimeApi *rime_api;
+
 typedef struct _IBusRimeEngine IBusRimeEngine;
 typedef struct _IBusRimeEngineClass IBusRimeEngineClass;
 
@@ -26,8 +28,8 @@ struct _IBusRimeEngineClass {
 
 /* functions prototype */
 static void ibus_rime_engine_class_init (IBusRimeEngineClass *klass);
-static void ibus_rime_engine_init (IBusRimeEngine *engine);
-static void ibus_rime_engine_destroy (IBusRimeEngine *engine);
+static void ibus_rime_engine_init (IBusRimeEngine *rime_engine);
+static void ibus_rime_engine_destroy (IBusRimeEngine *rime_engine);
 static gboolean ibus_rime_engine_process_key_event (IBusEngine *engine,
                                                     guint keyval,
                                                     guint keycode,
@@ -60,7 +62,7 @@ static void ibus_rime_engine_property_show (IBusEngine *engine,
 static void ibus_rime_engine_property_hide (IBusEngine *engine,
                                             const gchar *prop_name);
 
-static void ibus_rime_engine_update (IBusRimeEngine *rime);
+static void ibus_rime_engine_update (IBusRimeEngine *rime_engine);
 
 G_DEFINE_TYPE (IBusRimeEngine, ibus_rime_engine, IBUS_TYPE_ENGINE)
 
@@ -85,26 +87,25 @@ ibus_rime_engine_class_init (IBusRimeEngineClass *klass)
 }
 
 static void
-ibus_rime_create_session (IBusRimeEngine *rime)
+ibus_rime_create_session (IBusRimeEngine *rime_engine)
 {
-  rime->session_id = RimeCreateSession();
-  RimeSetOption(rime->session_id, "soft_cursor", True);
+  rime_engine->session_id = rime_api->create_session();
+  rime_api->set_option(rime_engine->session_id, "soft_cursor", True);
 }
 
 static void
-ibus_rime_engine_init (IBusRimeEngine *rime)
+ibus_rime_engine_init (IBusRimeEngine *rime_engine)
 {
-  //rime->session_id = RimeCreateSession();
-  ibus_rime_create_session(rime);
+  ibus_rime_create_session(rime_engine);
 
-  RIME_STRUCT_INIT(RimeStatus, rime->status);
-  RIME_STRUCT_CLEAR(rime->status);
+  RIME_STRUCT_INIT(RimeStatus, rime_engine->status);
+  RIME_STRUCT_CLEAR(rime_engine->status);
 
-  rime->table = ibus_lookup_table_new(9, 0, TRUE, FALSE);
-  g_object_ref_sink(rime->table);
+  rime_engine->table = ibus_lookup_table_new(9, 0, TRUE, FALSE);
+  g_object_ref_sink(rime_engine->table);
 
-  rime->props = ibus_prop_list_new();
-  g_object_ref_sink(rime->props);
+  rime_engine->props = ibus_prop_list_new();
+  g_object_ref_sink(rime_engine->props);
 
   IBusProperty* prop;
   IBusText* label;
@@ -120,7 +121,7 @@ ibus_rime_engine_init (IBusRimeEngine *rime)
                            TRUE,
                            PROP_STATE_UNCHECKED,
                            NULL);
-  ibus_prop_list_append(rime->props, prop);
+  ibus_prop_list_append(rime_engine->props, prop);
   label = ibus_text_new_from_static_string("部署");
   tips = ibus_text_new_from_static_string(_("Deploy"));
   prop = ibus_property_new("deploy",
@@ -132,7 +133,7 @@ ibus_rime_engine_init (IBusRimeEngine *rime)
                            TRUE,
                            PROP_STATE_UNCHECKED,
                            NULL);
-  ibus_prop_list_append(rime->props, prop);
+  ibus_prop_list_append(rime_engine->props, prop);
   label = ibus_text_new_from_static_string("同步");
   tips = ibus_text_new_from_static_string(_("Sync data"));
   prop = ibus_property_new("sync",
@@ -144,48 +145,48 @@ ibus_rime_engine_init (IBusRimeEngine *rime)
                            TRUE,
                            PROP_STATE_UNCHECKED,
                            NULL);
-  ibus_prop_list_append(rime->props, prop);
+  ibus_prop_list_append(rime_engine->props, prop);
 }
 
 static void
-ibus_rime_engine_destroy (IBusRimeEngine *rime)
+ibus_rime_engine_destroy (IBusRimeEngine *rime_engine)
 {
-  if (rime->session_id) {
-    RimeDestroySession(rime->session_id);
-    rime->session_id = 0;
+  if (rime_engine->session_id) {
+    rime_api->destroy_session(rime_engine->session_id);
+    rime_engine->session_id = 0;
   }
 
-  if (rime->status.schema_id) {
-    g_free(rime->status.schema_id);
+  if (rime_engine->status.schema_id) {
+    g_free(rime_engine->status.schema_id);
   }
-  if (rime->status.schema_name) {
-    g_free(rime->status.schema_name);
+  if (rime_engine->status.schema_name) {
+    g_free(rime_engine->status.schema_name);
   }
-  RIME_STRUCT_CLEAR(rime->status);
+  RIME_STRUCT_CLEAR(rime_engine->status);
 
-  if (rime->table) {
-    g_object_unref(rime->table);
-    rime->table = NULL;
+  if (rime_engine->table) {
+    g_object_unref(rime_engine->table);
+    rime_engine->table = NULL;
   }
 
-  if (rime->props) {
-    g_object_unref(rime->props);
-    rime->props = NULL;
+  if (rime_engine->props) {
+    g_object_unref(rime_engine->props);
+    rime_engine->props = NULL;
   }
 
-  ((IBusObjectClass *) ibus_rime_engine_parent_class)->destroy((IBusObject *)rime);
+  ((IBusObjectClass *) ibus_rime_engine_parent_class)->destroy(
+      (IBusObject *)rime_engine);
 }
 
 static void
 ibus_rime_engine_focus_in (IBusEngine *engine)
 {
-  IBusRimeEngine *rime = (IBusRimeEngine *)engine;
-  ibus_engine_register_properties((IBusEngine *)rime, rime->props);
-  if (!rime->session_id) {
-    //rime->session_id = RimeCreateSession();
-    ibus_rime_create_session(rime);
+  IBusRimeEngine *rime_engine = (IBusRimeEngine *)engine;
+  ibus_engine_register_properties(engine, rime_engine->props);
+  if (!rime_engine->session_id) {
+    ibus_rime_create_session(rime_engine);
   }
-  ibus_rime_engine_update(rime);
+  ibus_rime_engine_update(rime_engine);
 }
 
 static void
@@ -206,33 +207,34 @@ ibus_rime_engine_enable (IBusEngine *engine)
 static void
 ibus_rime_engine_disable (IBusEngine *engine)
 {
-  IBusRimeEngine *rime = (IBusRimeEngine *)engine;
-  if (rime->session_id) {
-    RimeDestroySession(rime->session_id);
-    rime->session_id = 0;
+  IBusRimeEngine *rime_engine = (IBusRimeEngine *)engine;
+  if (rime_engine->session_id) {
+    rime_api->destroy_session(rime_engine->session_id);
+    rime_engine->session_id = 0;
   }
 }
 
-static void ibus_rime_update_status(IBusRimeEngine *rime,
+static void ibus_rime_update_status(IBusRimeEngine *rime_engine,
                                     RimeStatus *status)
 {
   if (status &&
-          rime->status.is_disabled == status->is_disabled &&
-          rime->status.is_ascii_mode == status->is_ascii_mode &&
-          rime->status.schema_id && status->schema_id &&
-          !strcmp(rime->status.schema_id, status->schema_id)) {
-    return;  // no updates
+      rime_engine->status.is_disabled == status->is_disabled &&
+      rime_engine->status.is_ascii_mode == status->is_ascii_mode &&
+      rime_engine->status.schema_id && status->schema_id &&
+      !strcmp(rime_engine->status.schema_id, status->schema_id)) {
+    // no updates
+    return;
   }
 
-  rime->status.is_disabled = status ? status->is_disabled : False;
-  rime->status.is_ascii_mode = status ? status->is_ascii_mode : False;
-  if (rime->status.schema_id) {
-    g_free(rime->status.schema_id);
+  rime_engine->status.is_disabled = status ? status->is_disabled : False;
+  rime_engine->status.is_ascii_mode = status ? status->is_ascii_mode : False;
+  if (rime_engine->status.schema_id) {
+    g_free(rime_engine->status.schema_id);
   }
-  rime->status.schema_id =
+  rime_engine->status.schema_id =
       status && status->schema_id ? g_strdup(status->schema_id) : NULL;
 
-  IBusProperty* prop = ibus_prop_list_get(rime->props, 0);
+  IBusProperty* prop = ibus_prop_list_get(rime_engine->props, 0);
   const gchar* icon;
   IBusText* label;
   if (prop) {
@@ -261,42 +263,41 @@ static void ibus_rime_update_status(IBusRimeEngine *rime,
     }
     ibus_property_set_icon(prop, icon);
     ibus_property_set_label(prop, label);
-    ibus_engine_update_property((IBusEngine *)rime, prop);
+    ibus_engine_update_property((IBusEngine *)rime_engine, prop);
   }
 }
 
-static void ibus_rime_engine_update(IBusRimeEngine *rime)
+static void ibus_rime_engine_update(IBusRimeEngine *rime_engine)
 {
   // update properties
-
   RIME_STRUCT(RimeStatus, status);
-  if (RimeGetStatus(rime->session_id, &status)) {
-    ibus_rime_update_status(rime, &status);
-    RimeFreeStatus(&status);
+  if (rime_api->get_status(rime_engine->session_id, &status)) {
+    ibus_rime_update_status(rime_engine, &status);
+    rime_api->free_status(&status);
   }
   else {
-    ibus_rime_update_status(rime, NULL);
+    ibus_rime_update_status(rime_engine, NULL);
   }
 
   // commit text
-
   RIME_STRUCT(RimeCommit, commit);
-  if (RimeGetCommit(rime->session_id, &commit)) {
+  if (rime_api->get_commit(rime_engine->session_id, &commit)) {
     IBusText *text;
     text = ibus_text_new_from_string(commit.text);
-    ibus_engine_commit_text((IBusEngine *)rime, text);  // the text object will be released by ibus
-    RimeFreeCommit(&commit);
+    // the text object will be released by ibus
+    ibus_engine_commit_text((IBusEngine *)rime_engine, text);
+    rime_api->free_commit(&commit);
   }
 
   // begin updating UI
 
   RIME_STRUCT(RimeContext, context);
-  if (!RimeGetContext(rime->session_id, &context) ||
+  if (!rime_api->get_context(rime_engine->session_id, &context) ||
       context.composition.length == 0) {
-    ibus_engine_hide_preedit_text((IBusEngine *)rime);
-    ibus_engine_hide_auxiliary_text((IBusEngine *)rime);
-    ibus_engine_hide_lookup_table((IBusEngine *)rime);
-    RimeFreeContext(&context);
+    ibus_engine_hide_preedit_text((IBusEngine *)rime_engine);
+    ibus_engine_hide_auxiliary_text((IBusEngine *)rime_engine);
+    ibus_engine_hide_lookup_table((IBusEngine *)rime_engine);
+    rime_api->free_context(&context);
     return;
   }
 
@@ -304,25 +305,36 @@ static void ibus_rime_engine_update(IBusRimeEngine *rime)
   IBusText* text = NULL;
   guint inline_cursor_pos = 0;
   int offset = 0;
-  gboolean inline_preedit = g_ibus_rime_settings.embed_preedit_text && context.commit_text_preview;
-  gboolean highlighting = (context.composition.sel_start < context.composition.sel_end);
+  gboolean inline_preedit =
+      g_ibus_rime_settings.embed_preedit_text && context.commit_text_preview;
+  gboolean highlighting =
+      (context.composition.sel_start < context.composition.sel_end);
   if (inline_preedit) {
     inline_text = ibus_text_new_from_string(context.commit_text_preview);
     guint inline_text_len = ibus_text_get_length(inline_text);
     inline_cursor_pos = inline_text_len;
     inline_text->attrs = ibus_attr_list_new();
-    ibus_attr_list_append(inline_text->attrs,
-                          ibus_attr_underline_new(IBUS_ATTR_UNDERLINE_SINGLE, 0, inline_text_len));
+    ibus_attr_list_append(
+        inline_text->attrs,
+        ibus_attr_underline_new(
+            IBUS_ATTR_UNDERLINE_SINGLE, 0, inline_text_len));
     // hide converted range of auxiliary text if preedit is inline
     if (highlighting) {
       offset = context.composition.sel_start;
-      glong highlighting_start = g_utf8_strlen(context.composition.preedit, offset);
-      ibus_attr_list_append(inline_text->attrs,
-                            ibus_attr_foreground_new(g_ibus_rime_settings.color_scheme->text_color,
-                                                     highlighting_start, inline_text_len));
-      ibus_attr_list_append(inline_text->attrs,
-                            ibus_attr_background_new(g_ibus_rime_settings.color_scheme->back_color,
-                                                     highlighting_start, inline_text_len));
+      glong highlighting_start =
+          g_utf8_strlen(context.composition.preedit, offset);
+      ibus_attr_list_append(
+          inline_text->attrs,
+          ibus_attr_foreground_new(
+              g_ibus_rime_settings.color_scheme->text_color,
+              highlighting_start,
+              inline_text_len));
+      ibus_attr_list_append(
+          inline_text->attrs,
+          ibus_attr_background_new(
+              g_ibus_rime_settings.color_scheme->back_color,
+              highlighting_start,
+              inline_text_len));
     }
     else {
       offset = context.composition.length;  // hide auxiliary text
@@ -332,43 +344,55 @@ static void ibus_rime_engine_update(IBusRimeEngine *rime)
     const char* preedit = context.composition.preedit + offset;
     text = ibus_text_new_from_string(preedit);
     glong preedit_len = g_utf8_strlen(preedit, -1);
-    glong cursor_pos = g_utf8_strlen(preedit, context.composition.cursor_pos - offset);
+    glong cursor_pos =
+        g_utf8_strlen(preedit, context.composition.cursor_pos - offset);
     text->attrs = ibus_attr_list_new();
     if (highlighting) {
-      glong start = g_utf8_strlen(preedit, context.composition.sel_start - offset);
+      glong start = g_utf8_strlen(
+          preedit,context.composition.sel_start - offset);
       glong end = g_utf8_strlen(preedit, context.composition.sel_end - offset);
-      ibus_attr_list_append(text->attrs,
-                            ibus_attr_foreground_new(RIME_COLOR_BLACK, start, end));
-      ibus_attr_list_append(text->attrs,
-                            ibus_attr_background_new(RIME_COLOR_LIGHT, start, end));
+      ibus_attr_list_append(
+          text->attrs,
+          ibus_attr_foreground_new(RIME_COLOR_BLACK, start, end));
+      ibus_attr_list_append(
+          text->attrs,
+          ibus_attr_background_new(RIME_COLOR_LIGHT, start, end));
     }
   }
 
   if (inline_text) {
-    ibus_engine_update_preedit_text((IBusEngine *)rime, inline_text, inline_cursor_pos, TRUE);
+    ibus_engine_update_preedit_text(
+        (IBusEngine *)rime_engine, inline_text, inline_cursor_pos, TRUE);
   }
   else {
-    ibus_engine_hide_preedit_text((IBusEngine *)rime);
+    ibus_engine_hide_preedit_text((IBusEngine *)rime_engine);
   }
   if (text) {
-    ibus_engine_update_auxiliary_text((IBusEngine *)rime, text, TRUE);
+    ibus_engine_update_auxiliary_text((IBusEngine *)rime_engine, text, TRUE);
   }
   else {
-    ibus_engine_hide_auxiliary_text((IBusEngine *)rime);
+    ibus_engine_hide_auxiliary_text((IBusEngine *)rime_engine);
   }
 
-  ibus_lookup_table_clear(rime->table);
+  ibus_lookup_table_clear(rime_engine->table);
   if (context.menu.num_candidates) {
     int i;
-    int num_select_keys = context.menu.select_keys ? strlen(context.menu.select_keys) : 0;
-    gboolean has_labels = RIME_STRUCT_HAS_MEMBER(context, context.select_labels) && context.select_labels;
+    int num_select_keys =
+        context.menu.select_keys ? strlen(context.menu.select_keys) : 0;
+    gboolean has_labels =
+        RIME_STRUCT_HAS_MEMBER(context, context.select_labels) &&
+        context.select_labels;
     gboolean has_page_down = !context.menu.is_last_page;
-    gboolean has_page_up = context.menu.is_last_page && context.menu.page_no > 0;
-    ibus_lookup_table_set_round(rime->table, !(context.menu.is_last_page || context.menu.page_no == 0)); //show page up for middle page
-    ibus_lookup_table_set_page_size(rime->table, context.menu.page_size);
+    gboolean has_page_up =
+        context.menu.is_last_page && context.menu.page_no > 0;
+    ibus_lookup_table_set_round(
+        rime_engine->table,
+        !(context.menu.is_last_page || context.menu.page_no == 0));
+    ibus_lookup_table_set_page_size(rime_engine->table, context.menu.page_size);
     if (has_page_up) { //show page up for last page
       for (i = 0; i < context.menu.page_size; ++i) {
-        ibus_lookup_table_append_candidate(rime->table, ibus_text_new_from_string(""));
+        ibus_lookup_table_append_candidate(
+            rime_engine->table, ibus_text_new_from_string(""));
       }
     }
     for (i = 0; i < context.menu.num_candidates; ++i) {
@@ -389,7 +413,7 @@ static void ibus_rime_engine_update(IBusRimeEngine *rime)
       else {
         cand_text = ibus_text_new_from_string(text);
       }
-      ibus_lookup_table_append_candidate(rime->table, cand_text);
+      ibus_lookup_table_append_candidate(rime_engine->table, cand_text);
       IBusText *label = NULL;
       if (i < context.menu.page_size && has_labels) {
         label = ibus_text_new_from_string(context.select_labels[i]);
@@ -400,27 +424,32 @@ static void ibus_rime_engine_update(IBusRimeEngine *rime)
       else {
         label = ibus_text_new_from_printf("%d", (i + 1) % 10);
       }
-      ibus_lookup_table_set_label(rime->table, i, label);
+      ibus_lookup_table_set_label(rime_engine->table, i, label);
     }
     if (has_page_down) { //show page down except last page
-      ibus_lookup_table_append_candidate(rime->table, ibus_text_new_from_string(""));
+      ibus_lookup_table_append_candidate(
+          rime_engine->table, ibus_text_new_from_string(""));
     }
     if (has_page_up) { //show page up for last page
-      ibus_lookup_table_set_cursor_pos(rime->table, context.menu.page_size + context.menu.highlighted_candidate_index);
+      ibus_lookup_table_set_cursor_pos(
+          rime_engine->table,
+          context.menu.page_size + context.menu.highlighted_candidate_index);
     }
     else {
-      ibus_lookup_table_set_cursor_pos(rime->table, context.menu.highlighted_candidate_index);
+      ibus_lookup_table_set_cursor_pos(
+          rime_engine->table, context.menu.highlighted_candidate_index);
     }
-    ibus_lookup_table_set_orientation(rime->table, g_ibus_rime_settings.lookup_table_orientation);
-    ibus_engine_update_lookup_table((IBusEngine *)rime, rime->table, TRUE);
+    ibus_lookup_table_set_orientation(
+        rime_engine->table, g_ibus_rime_settings.lookup_table_orientation);
+    ibus_engine_update_lookup_table(
+        (IBusEngine *)rime_engine, rime_engine->table, TRUE);
   }
   else {
-    ibus_engine_hide_lookup_table((IBusEngine *)rime);
+    ibus_engine_hide_lookup_table((IBusEngine *)rime_engine);
   }
 
   // end updating UI
-
-  RimeFreeContext(&context);
+  rime_api->free_context(&context);
 }
 
 static gboolean
@@ -429,21 +458,21 @@ ibus_rime_engine_process_key_event (IBusEngine *engine,
                                     guint       keycode,
                                     guint       modifiers)
 {
-  IBusRimeEngine *rime = (IBusRimeEngine *)engine;
+  IBusRimeEngine *rime_engine = (IBusRimeEngine *)engine;
 
   modifiers &= (IBUS_RELEASE_MASK | IBUS_LOCK_MASK | IBUS_SHIFT_MASK |
                 IBUS_CONTROL_MASK | IBUS_MOD1_MASK | IBUS_SUPER_MASK);
 
-  if (!RimeFindSession(rime->session_id)) {
-    //rime->session_id = RimeCreateSession();
-    ibus_rime_create_session(rime);
+  if (!rime_api->find_session(rime_engine->session_id)) {
+    ibus_rime_create_session(rime_engine);
   }
-  if (!rime->session_id) {  // service disabled
-    ibus_rime_engine_update(rime);
+  if (!rime_engine->session_id) {  // service disabled
+    ibus_rime_engine_update(rime_engine);
     return FALSE;
   }
-  gboolean result = RimeProcessKey(rime->session_id, keyval, modifiers);
-  ibus_rime_engine_update(rime);
+  gboolean result =
+      rime_api->process_key(rime_engine->session_id, keyval, modifiers);
+  ibus_rime_engine_update(rime_engine);
   return result;
 }
 
@@ -452,26 +481,28 @@ static void ibus_rime_engine_property_activate (IBusEngine *engine,
                                                 guint prop_state)
 {
   extern void ibus_rime_start(gboolean full_check);
+  IBusRimeEngine *rime_engine = (IBusRimeEngine *)engine;
   if (!strcmp("deploy", prop_name)) {
-    RimeFinalize();
+    rime_api->finalize();
     ibus_rime_start(TRUE);
-    ibus_rime_engine_update((IBusRimeEngine *)engine);
+    ibus_rime_engine_update(rime_engine);
   }
   else if (!strcmp("sync", prop_name)) {
     // in the case a maintenance thread has already been started
-    // by RimeStartMaintenance(); the following call to RimeSyncUserData
+    // by start_maintenance(); the following call to sync_user_data()
     // will queue data synching tasks for execution in the working
     // maintenance thread, and will return False.
     // however, there is still chance that the working maintenance thread
     // happens to be quitting when new tasks are added, thus leaving newly
     // added tasks undone...
-    RimeSyncUserData();
-    ibus_rime_engine_update((IBusRimeEngine *)engine);
+    rime_api->sync_user_data();
+    ibus_rime_engine_update(rime_engine);
   }
   else if (!strcmp("InputMode", prop_name)) {
-    IBusRimeEngine *rime = (IBusRimeEngine *)engine;
-    RimeSetOption(rime->session_id, "ascii_mode", !RimeGetOption(rime->session_id, "ascii_mode"));
-    ibus_rime_engine_update(rime);
+    rime_api->set_option(
+        rime_engine->session_id, "ascii_mode",
+        !rime_api->get_option(rime_engine->session_id, "ascii_mode"));
+    ibus_rime_engine_update(rime_engine);
   }
 }
 
@@ -480,31 +511,32 @@ static void ibus_rime_engine_candidate_clicked (IBusEngine *engine,
                                                 guint button,
                                                 guint state)
 {
-  IBusRimeEngine *rime = (IBusRimeEngine *)engine;
-  RimeApi * additionalApis = rime_get_api();
-  if (RIME_API_AVAILABLE(additionalApis, select_candidate)) {
+  IBusRimeEngine *rime_engine = (IBusRimeEngine *)engine;
+  if (RIME_API_AVAILABLE(rime_api, select_candidate)) {
     RIME_STRUCT(RimeContext, context);
-    if (!RimeGetContext(rime->session_id, &context) ||
+    if (!rime_api->get_context(rime_engine->session_id, &context) ||
       context.composition.length == 0) {
-      RimeFreeContext(&context);
+      rime_api->free_context(&context);
       return;
     }
-    additionalApis->select_candidate(rime->session_id, context.menu.page_no * context.menu.page_size + index);
-    RimeFreeContext(&context);
-    ibus_rime_engine_update(rime);
+    rime_api->select_candidate(
+        rime_engine->session_id,
+        context.menu.page_no * context.menu.page_size + index);
+    rime_api->free_context(&context);
+    ibus_rime_engine_update(rime_engine);
   }
 }
 
 static void ibus_rime_engine_page_up (IBusEngine *engine)
 {
-    IBusRimeEngine *rime = (IBusRimeEngine *)engine;
-    RimeProcessKey(rime->session_id, IBUS_KEY_Page_Up, 0);
-    ibus_rime_engine_update(rime);
+    IBusRimeEngine *rime_engine = (IBusRimeEngine *)engine;
+    rime_api->process_key(rime_engine->session_id, IBUS_KEY_Page_Up, 0);
+    ibus_rime_engine_update(rime_engine);
 }
 
 static void ibus_rime_engine_page_down (IBusEngine *engine)
 {
-    IBusRimeEngine *rime = (IBusRimeEngine *)engine;
-    RimeProcessKey(rime->session_id, IBUS_KEY_Page_Down, 0);
-    ibus_rime_engine_update(rime);
+    IBusRimeEngine *rime_engine = (IBusRimeEngine *)engine;
+    rime_api->process_key(rime_engine->session_id, IBUS_KEY_Page_Down, 0);
+    ibus_rime_engine_update(rime_engine);
 }
